/* This file is part of dc_qt.

    dc_qt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    dc_qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dc_qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "dc_settings.h"
#include "dc_parse.h"
#include "util.h"
#include "debugdlg.h"

#include <qdir.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include <assert.h>
#include <qmessagebox.h>
#include <qdir.h>

#include <iostream>

using namespace std;


typedef pair<int,QString> Pair;
typedef pair<QString,QString> StrPair;

#define DEFAULT_HUBLIST "http://www.hublist.org/PublicHubList.xml.bz2"

// define SETTINGS_DEBUG

dc_settings::dc_settings()
{
}

int dc_settings::init()
{
    user_home_dir = QDir::homeDirPath(); // getenv("HOME");
    settings_file = user_home_dir + "/.dc_qt/dc_qt.conf";
    favourites_file = user_home_dir + "/.dc_qt/dc_qt.favourites";

    set_profile("default");

    set_setting("port","1412");
    set_setting("nick","dcqtuser");
    set_setting("description","dcqt user");
    set_setting("email","dcqtuser");
    set_setting("connection","Cable");
    set_setting("slots","3");
    set_setting("local_ip","");
    set_setting("font_encoding","CP1252");
    set_setting("allow_file_list_ul","true");
    set_setting("dl_rate_limit","0");
    set_setting("desc_tag","dctc");
    set_setting("same_hub_only","false");
    set_setting("passive_mode","false");
    set_setting("ul_rate_limit","0");
    set_setting("dl_dir","/tmp");
    set_setting("font","Arial,12,-1,5,50,0,0,0,0,0");
    set_setting("use_proxy","false");
    set_setting("use_socks","false");
    set_setting("tool_iconset","Default");
    set_setting("nick_highlight","true");
    set_setting("chat_timestamp","true");
    
    QDir dc_dir( user_home_dir + "/.dc_qt" );
    if( !dc_dir.exists() )
    {
        dc_dir.mkdir( user_home_dir + "/.dc_qt" );
        return NO_SETTINGS;
    }

    if(!load_from_file())
        return ERROR_LOADING_PREFS;

    if(!load_favourites_file())
        return ERROR_LOADING_FAVOURITES;

    return SUCCESS;
}


void dc_settings::set_profile(const QString &name)
{
    current_profile = name;
}

QString dc_settings::get_setting(const QString& key)
{
    return get_setting(current_profile,key);
}

QString dc_settings::get_setting(const QString& profile, const QString& key)
{
  if(settings.find(profile)!=settings.end()) {
    if(settings[profile].find(key)!=settings[profile].end()) {
      return settings[profile][key];
    }
  }
  return QString("");
}

void dc_settings::set_setting(const QString& key, const QString &value)
{
  set_setting(current_profile,key,value);
}

void dc_settings::set_setting(const QString& profile,const QString& key, const QString &value)
{
  settings[current_profile][key] = value;
}

bool dc_settings::load_from_file()
{
    // Default values goes here in case they're not in the file.
    QFile file(settings_file);

    if (file.open(IO_ReadOnly))
    {
        QTextStream stream(&file);
        QString inbuf;
        QString profile;

        while (!stream.eof())
        {
            inbuf = stream.readLine();

            if(inbuf.startsWith("\\profile"))
	      profile = inbuf.section(" ",1,1);
            else
            {
                if(profile.isEmpty())
                    continue;
                int pos = inbuf.find("=");
                if(pos <= 0)
                    continue;
                set_setting(profile,inbuf.left(pos),inbuf.right(inbuf.length()-pos-1));
            }
        }
    }
    else
        return false;
    return true;
}

// this is currently broken.  A hub that has an '=' in it will not be
// read in correctly.
bool dc_settings::load_favourites_file()
{
    QFile file(favourites_file);

    if( file.open(IO_ReadOnly) )
    {
        QTextStream stream(&file);
        QString inbuf;
        QString name,ip;
        QString pwd = QString::null;

        while( !stream.eof() )
        {
            inbuf = stream.readLine();
            if( inbuf[0] == '#' )
                continue;
            dc_parse p(inbuf,'=');
            if( p.get_str(0) == "name" )
                name = p.get_str(1);
            else if( p.get_str(0) == "address" )
            {
                ip = p.get_str(1);
            }
            else if( p.get_str(0) == "password")
            {
                pwd  = p.get_str(1);
                if(pwd=="<void>")
                    pwd = QString::null;
                favourites.insert(pair<QString,FavData>(name,FavData(ip,pwd)));
            }
            else if( p.get_str(0) == "hublisturl")
            {
                hubLists+=p.get_str(1);
            }
        }
        if(!hubLists.contains(DEFAULT_HUBLIST))
            hubLists+=DEFAULT_HUBLIST;
        return true;
    }
    else
        return false;
}

bool dc_settings::save_to_file()
{
    QFile file(settings_file);

    if(file.open(IO_WriteOnly))
    {
        QTextStream stream(&file);

        for(ProfileMap::iterator ik = settings.begin();
	    ik != settings.end();
	    ++ik) {
	    stream << "\\profile " << ik.key() << "\n";

	    for(KeyMap::iterator iv = ik.data().begin();
		iv != ik.data().end();
		++iv) {
	      stream << iv.key() << "=" << iv.data() << "\n";
	    }

	}
	
	file.close();
        return true;
    }
    return false;
}


void dc_settings::add_favourite(const QString &name,const QString &address,const QString &password)
{
    favourites.erase(name);
    favourites.insert( pair<QString,FavData>( name, FavData(address,password) ) );
}

void dc_settings::del_favourite(const QString &name)
{
    favourites.erase(name);
}

bool dc_settings::get_favourite(int i,QString &name, QString &address, QString &password)
{
    StrMap::iterator it = favourites.begin();
    while(i--)
        it++;
    if(it==favourites.end())
        return false;

    name = (*it).first;
    address = (*it).second.ip;
    password = (*it).second.pwd;
    return true;
}

const hub_info dc_settings::get_favourite(const QString &name)
{
    StrMap::iterator it = favourites.find(name);
    if( it == favourites.end() )
        return hub_info("", "", "", "");
    else
        return hub_info(name, (*it).second.ip, "", "", (*it).second.pwd);
}


bool dc_settings::save_favourites_file()
{
    QFile file(favourites_file);
    if( file.open(IO_WriteOnly) )
    {
        QTextStream stream(&file);
        QString name,ip,pwd;
        stream << "# dc_qt favourite file. Generated automatically, DO NOT EDIT.\n";

        for(int i=0;i < num_favourites();i++)
        {
            get_favourite(i,name,ip,pwd);
            if(pwd.isEmpty())
                pwd = "<void>";
            stream << "name=" << name << "\naddress=" << ip << "\npassword=" << pwd << "\n";
        }
        for(unsigned int i=0;i<hubLists.count();i++)
            stream << "hublisturl=" << hubLists[i] << "\n";
        return true;
    }
    return false;
}

QStringList dc_settings::get_connection_types()
{
    QStringList strlst("LAN(T3)");
    strlst << "LAN(T1)";
    strlst << "Cable";
    strlst << "DSL";
    strlst << "ISDN";
    strlst << "Sattelite";
    strlst << "56Kbps";
    strlst << "33.6Kbps";
    strlst << "28.8Kbps";
    return strlst;
}

